#!/usr/bin/perl
#
# Script to create parameter files from C1.scm

use strict;

my $paramfl = "C1.scm";

mkdir("epsilon",0777);
mkdir("extended",0777);
mkdir("interval",0777);
mkdir("kahan",0777);
mkdir("nothing",0777);
mkdir("offset",0777);
mkdir("offsetwgt0",0777);
mkdir("string",0777);
mkdir("warn",0777);

# Set up parameter files for contrasting the following classes of Agents:
# SSATotal and SSAMean.
# value for subdir key => [Class, [class parameters], initial wealth]

my %dirsettingsptSSATotal
  = ("epsilon" => ["DoubleEpsilon", ['~0.0', '~4.99e-1', '~5.00e-1'], '0.0'],
     "extended" => ["DoubleLong", [], '0.0'],
     "interval" => ["DoubleIntervalWarn", [], '0.0'],
     "kahan" => ["DoubleKahanSum", [], '0.0'],
     "nothing" => ["DoubleSimple", [], '0.0'],
     "offset" => ["DoubleOffset", ['~0.0', '~2.251799813685248e+15',
				   '~4.503599627370496e+15'], '0.0'],
     "offsetwgt0" => ["DoubleOffset", ['~0.0', '~2.251799813685248e+15',
				   '~4.503599627370496e+15'], '3.0'],
     "string" => ["DoubleString", ['s3', 's4', 's25', 's50'], '0.0'],
     "warn" => ["DoubleWarn", ['!ALL'], '0.0']);

my %dirsettingsptSSAMean
  = ("epsilon" => ["DoubleEpsilon", ['~0.0', '~5.55e-2', '~5.56e-2'], '0.0'],
     "extended" => ["DoubleLong", [], '0.0'],
     "interval" => ["DoubleIntervalWarn", [], '0.0'],
     "kahan" => ["DoubleKahanSum", [], '0.0'],
     "nothing" => ["DoubleSimple", [], '0.0'],
     "offset" => ["DoubleOffset", ['~0.0', '~2.81474976710656e+14',
				   '~5.62949953421312e+14'], '0.0'],
     "offsetwgt0" => ["DoubleOffset", ['~0.0', '~2.81474976710656e+14',
				   '~5.62949953421312e+14'], '3.0'],
     "string" => ["DoubleString", ['s3', 's4', 's25', 's50'], '0.0'],
     "warn" => ["DoubleWarn", ['!ALL'], '0.0']);

foreach my $Gsettings ([\%dirsettingsptSSATotal, "SSATotal"],
		       [\%dirsettingsptSSAMean, "SSAMean"])
{
  my $settings = $$Gsettings[0];
  my $agentClass = $$Gsettings[1];

  print "Creating parameter files for Agent Class = $agentClass\n";
  while(my ($subdir, $sdsettings) = each(%$settings)) {
    my $class = $$sdsettings[0];
    my $classparams = $$sdsettings[1];
    my $initwealth = $$sdsettings[2];
    
    print "Creating parameter files in subdir $subdir\n";
    if($#$classparams == -1) {
      &create_param_fl($paramfl, $subdir, $class, "0", $agentClass,
		       $initwealth);
      next;
    }
    elsif($#$classparams == 0) {
      &create_param_fl($paramfl, $subdir, $class, $$classparams[0], 
		       $agentClass, $initwealth);
      next;
    }
    foreach my $classparam (@$classparams) {
      &create_param_fl($paramfl, $subdir, $class, $classparam, $agentClass,
		       $initwealth);
    }
  }
}

print "Done\n";
exit 0;

sub create_param_fl {
  my ($orig, $subdir, $class, $cparam, $agentClass, $w0) = @_;

  open(ORIG, "<$orig") or die "Error reading base parameter file $orig: $!\n";

  my $newfl = "$agentClass";
  if($cparam ne "0") {
    my $i = 1;

    while(-e "$subdir/$newfl-$i.scm") {
      $i++;
    }
    $newfl .= "-$i";
  }
  $newfl .= ".scm";

  if(-e "$subdir/$newfl") {
    warn "New parameter file $subdir/$newfl unexpectedly exists\n";
  }

  open(NEW, ">$subdir/$newfl")
    or die "Error creating new parameter file $subdir/$newfl: $!\n";

  while(my $line = <ORIG>) {
    chomp $line;
    if(($line !~ /^\s*\#/ || $line !~ /DoubleWarn/ )
       && ($line !~ /^\s*\#/ || $line !~ /agentClassStr/)) 
    {
      print NEW "$line\n";
    }
    elsif($line =~ /^(.*:)(\S+)\s+DoubleWarn=([^!]+)(!ALL)?\s*$/) {
      my $pre = $1;
      my $modelparam = $2;
      my $paramvalue = $3;

      print NEW "${pre}${modelparam} ${class}=";
      if($modelparam eq "initialWealthStr") {
	print NEW "$w0";
      }
      else {
	print NEW "$paramvalue";
      }
      if($cparam ne "0") {
	print NEW "$cparam";
	$cparam = "0";
      }
      print NEW "\n";
    }
    elsif($line =~ /^(.*agentClassStr)/){
	my $preAg = $1;
	print NEW "${preAg} ${agentClass}\n";
    }
    else {
      die "Don't know what to do with this line in base parameter file $orig".
	":\n$line\n";
    }
  }
  print "Created parameter file $subdir/$newfl\n";
  close(NEW);
  close(ORIG);
}

